/*
** Démo d'envoi de données JSON sur l'interface USB
**
*/

#include <avr/wdt.h>               // Pour le WatchDog

int coef = 2;
int decal = 0;

void setup() { 
  // Initialisation du WatchDog
  wdt_enable(WDTO_8S);

  pinMode(LED_BUILTIN, OUTPUT);
  
  Serial.begin(115200); 
  Serial.println(""); 
  Serial.println("Arduino initialized !"); 
  Serial.println(""); 
}

void loop() {
 
  // Reset du watchDog : C'est reparti pour 8 secondes
  wdt_reset();

  int d;
  d = analogRead(A0);
  Serial.println(d);
  jsonBegin();
  jsonAddInt(false, "AAA", d);
  jsonAddInt(true, "BBB", d*coef + decal);
  jsonAddFloat(true, "CCC", (float)d/2.0);
  jsonEnd();
  Serial.println("loop");
  serialGet();

  digitalWrite(LED_BUILTIN, HIGH);   // Allume la LED ...
  delay(200);                        // ...pendant 200 milli secondes
  digitalWrite(LED_BUILTIN, LOW);    // Eteint la LED ...
  delay(100);                        // ...pendant &00 milli secondes
  digitalWrite(LED_BUILTIN, HIGH);   // Allume la LED ...
  delay(200);                        // ...pendant 200 milli secondes
  digitalWrite(LED_BUILTIN, LOW);    // Eteint la LED
  delay(1500);
}

void jsonBegin() {
  Serial.print("{");
}

void jsonEnd() {
    Serial.println("}");
}

void jsonAddSeparator(bool withSeparator) {
  if (withSeparator == true) {
    Serial.print(","); 
  }
}

void jsonAddName(String name) {
  Serial.print("\"");
  Serial.print(name);
  Serial.print("\":");
}

void jsonAddInt(bool withSeparator, String name, int val) {
  jsonAddSeparator(withSeparator);
  jsonAddName(name);
  Serial.print(val);
}

void jsonAddFloat(bool withSeparator, String name, float val) {
  jsonAddSeparator(withSeparator);
  jsonAddName(name);
  Serial.print(val);
}

// Lire les éventuelles commandes reçues sur le port série et y répondre
void serialGet(void) {
  String commandText = String("");
  char byteInSerial;
  while (Serial.available() > 0) {
    byteInSerial = Serial.read();
    // On teste la fin de ligne
    if ( (byteInSerial == 10) || (byteInSerial == 13) ) {
      commandProcess(commandText);
    }
    else {
      commandText += String(byteInSerial);
    }
  }
}  

void commandProcess(String commandTxt) {
  Serial.println("commandProcess");
  // Test les différentes commandes possibles
  if (commandTxt.startsWith("Reset")) {
    commandReset();
  }
  else if (commandTxt.startsWith("fort")) {
    coef = 8;
  }
  else if (commandTxt.startsWith("normal")) {
    coef = 2;
    decal = 0;
  }
  else if (commandTxt.startsWith("decal") && commandTxt.length() > 6) {
    String d = commandTxt.substring(6);
    //if (d.
    decal = d.toInt(); 
  }
  else if (commandTxt.startsWith("toto")) {
    Serial.println(" fait du velo !");
  }
  else if (commandTxt.startsWith("papa")) {
    Serial.println(" fait du chocolat !");
  }
  else {
    commandUnknown(commandTxt);
  }
}

// Commande de reset
void commandReset() {
  Serial.println("commandReset");
  delay(50);
  software_Reset();
}
 
void software_Reset() { // Restarts program from beginning but does not reset the peripherals and registers
  // Deux possibilité :
  
  // 1) Sauter au début de la mémoire, Fonctionne en général sur tous les Arduino
  asm volatile ("  jmp 0");  
  /*
  // 2) ou Demander au watchdog de ne pas attendre plus que 15 millisecondes avant d'agir, Ne fonctionne pas sur les premières versions du bootloader
  wdt_enable(WDTO_15MS);
  // Demander une longue attente pour déclencher le watchdog
  delay(10000);
  */
} 
 
// Commande inconnue !
void commandUnknown(String commandTxt) {
  Serial.print("commandUnknown:");
  Serial.println(commandTxt + "");
  delay(50);
}
 


